// fireworks.java

import java.awt.*;
import java.applet.Applet;

public class fireworks extends Applet implements Runnable
{
   private rocket [] r = new rocket [30];

   private int width;
   private int height;

   private Graphics bkG;
   private Image bkIm;

   private Thread animateThread;

   public void init ()
   {
      // Get the applet's draw-able area width and size.

      width = getSize ().width;
      height = getSize ().height;

      // Create a double buffer and obtain its graphics context.

      bkIm = createImage (width, height);
      bkG = bkIm.getGraphics ();
   }

   public void start ()
   {
      // If there is no animation thread ...

      if (animateThread == null)
      {
          // Create and start an animation thread.

          animateThread = new Thread (this);
          animateThread.start ();
      }
   }

   public void run ()
   {
      // While the applet has not been stopped ...

      while (animateThread != null)
      {
         // Obtain the current system time in milliseconds.

         long time = System.currentTimeMillis ();

         try
         {
            // Wake me up no more than 100 milliseconds from
            // now.

            time += 100;
            Thread.sleep 
                (Math.max (0,
                           time - System.currentTimeMillis ()));
         }
         catch (InterruptedException e) {}

         // Update the applet's draw-able area.

         repaint ();
      }
   }

   public void update (Graphics g)
   {
      // Re-paint the applet's draw-able area without erasing
      // the background.  This reduces flicker.

      paint (g);
   }

   public void paint (Graphics g)
   {
      // Set the double buffer's pixels to black.

      bkG.setColor (Color.black);
      bkG.fillRect (0, 0, width, height);

      // For all rockets ...

      for (int i = 0; i < r.length; i++)

           // If a rocket exists, move and draw that rocket.

           if (r [i] != null)
               r [i] = r [i].moveAndDraw (bkG);
           else
           {
               // Either a rocket did not exist or it has blown
               // up.  Therefore, create a new rocket with a
               // random color and let this rocket "know" its
               // range of movement.

               Color c = new Color (rnd (256),      // red
                                    rnd (256),      // green
                                    rnd (256));     // blue

               r [i] = new rocket (c, width, height);
           }

      // Draw the next frame of rockets and explosions.

      g.drawImage (bkIm, 0, 0, this);
   }

   public void stop ()
   {
      // Kill the animation thread.

      animateThread = null;
   }

   private int rnd (int n)
   {
      // Return a random integer greater than or equal to 0
      // but less than n.

      return (int) (Math.random () * n);
   }
}

class rocket
{
   // rColor is the rocket's color.

   private Color rColor;

   // fragColor is the color of rocket fragments.

   private Color fragColor;

   // width and height describe the limits of the rocket's
   // range.

   private int width;
   private int height;

   // xPos and yPos identify the rocket's current position.

   private int xPos, yPos;

   // xVel and yVel identify the rocket's current velocity.

   private double xVel, yVel;

   // counter identifies the number of times moveAndDraw
   // (Graphics) has been called since the rocket started to
   // explode.

   private int counter;

   // explosionExpansionRate identifies the rate of rocket
   // debris expansion.

   private double explosionExpansionRate;

   // maxExplosionRadius identifies the maximum size of the
   // rocket debris field.

   private double maxExplosionRadius =
                               50 + (int) (Math.random () * 50);

   // The following field is pre-calculate for performance
   // reasons.

   private double temp = 4.0 * maxExplosionRadius;

   rocket (Color rColor, int width, int height)
   {
      this.rColor = fragColor = rColor;
      this.width = width;
      this.height = height;

      // The rocket's initial horizontal position is somewhere
      // along the applet's horizontal width.

      xPos = (int) (Math.random () * width);

      // The rocket's intial vertical position is at the bottom
      // of the applet.

      yPos = height;

      // The rocket's initial horizontal velocity ranges from
      // -5.0 to 4.99999etc.

      xVel = -5 + Math.random () * 10;

      // The rocket's initial vertical velocity ranges from
      // -6.0 to -11.99999etc.

      yVel = -6 - Math.random () * 6;

      // The explosion has not yet begun.

      counter = 0;
      explosionExpansionRate = 0.0;
   }

   rocket moveAndDraw (Graphics g)
   {
      // Move the rocket.

      xPos += xVel;
      yPos += yVel;

      // Calculate the rocket.

      xVel /= 1.05;
      yVel += 0.25;

      // If the vertical velocity is less than -1, keep drawing
      // the rocket.

      if (yVel < -1)
      {
          // Rocket should be darker than initial explosion
          // ring.

          g.setColor (rColor.darker ());
          g.drawLine (xPos, yPos, xPos, yPos);
      }
      else
      {
          // Explosion has started.

          counter++;

          // Set the color of rocket fragments.

          g.setColor (fragColor);

          // Every four calls to moveAndDraw (Graphics) causes
          // the fragments color to darken as the explosion
          // ages.

          if (counter % 4 == 0)
              fragColor = fragColor.darker ();

          // Draw six explosion rings.  Each ring is slightly
          // offset from the prevous ring.

          for (int i = 0; i < 6; i++)
          {
               // Darken every third explosion ring.

               if (i % 3 == 2)
                   g.setColor (fragColor.darker ());

               // Calculate the expansion rate of the explosion.

               explosionExpansionRate +=  counter / temp;

               // Calculate the current radius of the debris
               // field.

               double radius = maxExplosionRadius *
                               explosionExpansionRate;

               // Draw the debris field for each ring.

               for (double angle = 0.0; angle < 2 * Math.PI;
                    angle += 0.5)
               {
                    double dx = Math.cos (angle) * radius;
                    double dy = Math.sin (angle) * radius;

                    int x = (int) (xPos + dx);
                    int y = (int) (yPos + dy);
  
                    g.drawLine (x, y, x, y);
               }

               // When the debris field size exceeds the maximum
               // radius of the debris field then terminate the
               // exploded rocket.

               if (radius > maxExplosionRadius)
                   return null;
          }
      }

      return this;
   }
}
