// jpad.java

import java.awt.*;
import java.awt.event.*;

class jpad extends Frame implements ActionListener, ItemListener
{
   private drawingSurface ds;

   public jpad ()
   {
      // Assign "Drawing Pad" title to title bar.

      super ("Drawing Pad");

      // Add a listener that responds to window closing
      // events.  When this event occurs (by clicking on the
      // close box in the title bar), exit program.

      addWindowListener (new WindowAdapter ()
                         {
                             public void windowClosing
                                           (WindowEvent e)
                             {
                                System.exit (0);
                             }
                         });

      // Create a control panel with a border layout and set its
      // background to light gray (to distinguish it from the
      // drawing surface.)

      Panel cp = new Panel ();
      cp.setLayout (new BorderLayout ());
      cp.setBackground (Color.lightGray);

      // Create a panel for the Clear button (to allow the
      // Clear button to maintain a smaller rectangular shape).

      Panel p = new Panel ();

      // Create a Clear button for clearing the drawing pad.

      Button b = new Button ("Clear");

      // Add the clear button to the control panel.

      p.add (b);

      // Register the current object as a listener to action
      // events that are generated when the Clear button is
      // pressed.

      b.addActionListener (this);

      // Add the Clear button panel to the control panel.

      cp.add ("West", p);

      // Create a panel for the color button group.

      p = new Panel ();

      // Create a check box group to represent the color button
      // group.

      CheckboxGroup cbg = new CheckboxGroup ();

      // Create the Red checkbox and add it to the check box
      // group.

      Checkbox cb = new Checkbox ("Red", true, cbg);

      // Add an item listener to respond to the user selecting
      // this checkbox.

      cb.addItemListener (this);

      // Add checkbox to panel.

      p.add (cb);

      // Create the Green checkbox and add it to the check box
      // group.

      cb = new Checkbox ("Green", false, cbg);

      // Add an item listener to respond to the user selecting
      // this checkbox.

      cb.addItemListener (this);

      // Add checkbox to panel.

      p.add (cb);

      // Create the Blue checkbox and add it to the check box
      // group.

      cb = new Checkbox ("Blue", false, cbg);

      // Add an item listener to respond to the user selecting
      // this checkbox.

      cb.addItemListener (this);

      // Add checkbox to panel.

      p.add (cb);

      // Add the color button group panel to the control panel.

      cp.add ("East", p);

      // Place the control panel panel in the north part of the
      // frame window.

      add ("North", cp);

      // Set the size of the frame window container to 600
      // pixels horizontally by 400 pixels vertically.

      setSize (600, 400);

      // Create a drawingSurface component with the width and
      // height of the frame window.

      ds = new drawingSurface (getSize ().width,
                               getSize ().height);

      // Place the drawingSurface component in the south part of
      // the frame window.

      add ("South", ds);

      // Do not allow the user to resize the frame window.

      setResizable (false);

      // Make sure that the frame window is visible.

      setVisible (true);
   }

   public void actionPerformed (ActionEvent e)
   {
      // If the user pressed the clear button, clear
      // the drawing pad area (the portion of the window
      // below the control panel).  Technically, we don't
      // have to use the If statement to see what button
      // was pressed because there is only one button.
      // However, we may wish to add other buttons down
      // the road so we'll keep the If statement.

      if (e.getActionCommand ().equals ("Clear"))
      {
          ds.clear ();
      }
   }

   public void itemStateChanged (ItemEvent e)
   {
      // Based on the selected checkbox, set the new drawing
      // color.

      if (e.getItem ().equals ("Red"))
          ds.setColor (Color.red);
      else
      if (e.getItem ().equals ("Green"))
          ds.setColor (Color.green);
      else
          ds.setColor (Color.blue);
   }

   public static void main (String [] args)
   {
      new jpad ();
   }
}

class drawingSurface extends Canvas
{
   // Mouse coordinates when mouse button pressed.

   private int oldX, oldY;

   // Current drawing color (default is red).

   private Color drawColor = Color.red;

   drawingSurface (int width, int height)
   {
      // Add a listener for mouse pressed events.  When the mouse
      // button is pressed, this signals that drawing is about to
      // begin.  Therefore, record the current mouse coordinates
      // to indicate where drawing will begin.

      addMouseListener (new MouseAdapter ()
                        {
                            public void mouseEntered
                                                   (MouseEvent e)
                            {
                               setCursor (new
                               Cursor (Cursor.CROSSHAIR_CURSOR));
                            }

                            public void mouseExited
                                                   (MouseEvent e)
                            {
                               setCursor (new
                               Cursor (Cursor.DEFAULT_CURSOR));
                            }

                            public void mousePressed
                                                   (MouseEvent e)
                            {
                               oldX = e.getX ();
                               oldY = e.getY ();
                            }
                        });

      // Add a listener for mouse drag events.  As long as a
      // mouse button is pressed, mouse drag events occur when
      // the mouse is moved.  For each mouse drag event, draw a
      // line from the old mouse coordinates to the new mouse
      // coordinates (as indicated by the mouse drag event).
      // Once drawing is done, update the old mouse coordinates
      // as the new coordinates.

      addMouseMotionListener (new MouseMotionAdapter ()
                              {
                                  public void mouseDragged
                                                   (MouseEvent e)
                                  {      
                                     Graphics g = getGraphics ();
                                     g.setColor (drawColor);
                                     g.drawLine (oldX, oldY,
                                                 e.getX (),
                                                 e.getY ());

                                     oldX = e.getX ();
                                     oldY = e.getY ();
                                  }
                              });

      setSize (width, height);
      setVisible (true);
  }

   void clear ()
   {
      Graphics g = getGraphics ();
      g.clearRect (0, 0, getSize ().width, getSize ().height);
   }

   void setColor (Color c)
   {
      drawColor = c;
   }
}
