// jed.java

// Java EDitor

import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class jed extends Frame
       implements ActionListener, TextListener
{
   // Program title - displayed in the Frame window's title bar.

   private final static String TITLE = "Java Editor";

   // File contents and status components.

   private TextArea fileContents;
   private Label status;

   // Name of current file, initialized to "" to indicate that
   // there is no current file.

   private String curfile = "";

   /*
      Note: When a file is loaded, curFile refers to the name of
      this file as the name of the current file.  When the user
      selects Save from the menu, jed can save the changed
      contents to this file.  When the user selects Open... from
      the menu, jed resets curfile to the name of this file - if
      it is successfully opened and its contents loaded into the
      file contents component.  When the user selects Save As...
      from the menu, jed resets curfile to the name of this file
      - if it is successfully created and the file contents
      component's contents successfully saved to this file.
   */

   // Text changed flag (true if changed).

   private boolean changed;

   /*
      Note: The changed flag defaults to false because there
      either is no loaded file when jed starts running or a file
      has just been loaded.  In either case, the user has not
      typed anything at this point.

      The changed flag is set to true in response to the user
      typing in the file contents component.  This flag is set
      to false when the file contents component's contents are
      successfully saved by a Save or Save As... menu command.

      If changed is true, the Open... menu command will prompt
      for changes to be saved before it will open a file.
   */

   // Default constructor.

   jed ()
   {
      // Call other constructor.  Pass "" to inform this
      // constructor that a file is not being loaded.

      this ("");
   }

   // Constructor that creates GUI and optionally loads file.

   jed (String filename)
   {
      // Set the title of the frame window.

      super (TITLE);

      // Create a TextArea component object with an initial
      // size of 80 columns by 25 rows.  Assign its reference
      // to fileContents.  This component will hold file text
      // from a previously loaded file or text that must be
      // saved to a file.

      fileContents = new TextArea (25, 80);

      // Add the file contents component to the north part of
      // the frame window.

      add ("North", fileContents);

      // Inform fileContents that the current jed object will
      // listen for text events from this component.  When a
      // text event occurs, the textValueChanged (TextEvent)
      // instance method declared in the jed class will be
      // called to handle this event.

      fileContents.addTextListener (this);

      // Create a Label component object with no text.  Assign
      // its reference to status.  This component will display
      // status messages.

      status = new Label ();

      // Add the status component to the south part of the frame
      // window.

      add ("South", status);

      // Create a File menu component.  This component will hold
      // various file-related components that do file-related
      // tasks.

      Menu m = new Menu ("File");

      // Create an Open... menu item component.  This component
      // will be used as the starting point for the file open
      // task.

      MenuItem mi = new MenuItem ("Open...");

      // Inform the Open... menu item component that the current
      // jed object will listen for action events from this
      // component.  When an action event occurs, the
      // actionPerformed (ActionEvent) instance method declared
      // in the jed class will be called to handle this event.

      mi.addActionListener (this);

      // Add the Open... menu item component to the File menu
      // component.

      m.add (mi);

      // Create a Save menu item component.  This component will
      // be used as the starting point for the file save task.

      mi = new MenuItem ("Save");

      // Make sure the Save menu item component is initially
      // disabled.  At this point, there is no text to save.

      mi.setEnabled (false);

      // Inform the Save menu item component that the current
      // jed object will listen for action events from this
      // component.  When an action event occurs, the
      // actionPerformed (ActionEvent) instance method declared
      // in the jed class will be called to handle this event.

      mi.addActionListener (this);

      // Add the Save menu item component to the File menu
      // component.

      m.add (mi);

      // Create a Save As... menu item component.  This
      // component will be used as the starting point for the
      // file save as task.

      mi = new MenuItem ("Save As...");

      // Make sure the Save As... menu item component is
      // initially disabled.  At this point, there is no text to
      // save.

      mi.setEnabled (false);

      // Inform the Save As... menu item component that the
      // current jed object will listen for action events from
      // this component.  When an action event occurs, the
      // actionPerformed (ActionEvent) instance method declared
      // in the jed class will be called to handle this event.

      mi.addActionListener (this);

      // Add the Save As... menu item component to the File menu
      // component.

      m.add (mi);

      // Create an Exit menu item component.  This component
      // will be used as the starting point for the program exit
      // task.

      mi = new MenuItem ("Exit");

      // Inform the Exit menu item component that the current 
      // jed object will listen for action events from this
      // component.  When an action event occurs, the
      // actionPerformed (ActionEvent) instance method declared
      // in the jed class will be called to handle this event.

      mi.addActionListener (this);

      // Add the Exit menu item component to the File menu
      // component.

      m.add (mi);

      // Create a menu bar component.

      MenuBar mb = new MenuBar ();

      // Add the File menu component to the menu bar component.

      mb.add (m);

      // Set the frame window's menu bar to the new menu bar.

      setMenuBar (mb);

      // Add a window listener object that responds to the user
      // pressing the close box on the title bar by quiting the
      // program.

      addWindowListener (new WindowAdapter ()
                         {
                             public void windowClosing
                                           (WindowEvent e)
                             {
                                // There is no point in exiting
                                // if there is text in the file
                                // contents component that must
                                // be saved.

                                if (changed)
                                {
                                    status.setText ("Please " +
                                                    "save " +
                                                    "file " +
                                                    "contents.");
                                    return;
                                }

                                System.exit (0);
                             }
                         });

      // If a non-empty string file name was passed as an
      // argument, attempt to load the file identified by this
      // name.  If the file is successfully loaded, update the
      // frame window's title bar and identify this file as the
      // currently loaded file.  Also, ensure that the Save and
      // Save As... menu item components are enabled.

      if (!filename.equals (""))
          if (load (filename))
          {
              setTitle (TITLE + ": " + filename);
              curfile = new String (filename);

              m = getMenuBar ().getMenu (0);
              m.getItem (1).setEnabled (true);
              m.getItem (2).setEnabled (true);
          }

      // Resize the frame window to the preferred size of its
      // components.

      pack ();

      // Assign focus to the file contents component.

      fileContents.requestFocus ();

      // Make the frame window visible to the user.

      setVisible (true);
   }

   // Application entry point method.

   public static void main (String [] args)
   {
      jed myJED;

      // If one argument was passed to jed, this argument is
      // the name of a file.  Therefore, create a jed object
      // and load this file.  If no arguments or more than
      // one argument was passed, either the user does not
      // want to start this editor with a pre-loaded file or
      // the user is goofing off.  Therefore, create a jed
      // object without a pre-loaded file.

      if (args.length == 1)
          myJED = new jed (args [0]);
      else
          myJED = new jed ();
   }

   // Event handler for menu item selections.

   public void actionPerformed (ActionEvent e)
   {
      int mode;
      String title;

      // If the user selected Exit from the menu, do the right
      // thing and exit.

      if (e.getActionCommand ().equals ("Exit"))
      {
          // There is no point in exiting if there is text
          // in the file contents component that must be
          // saved.

          if (changed)
          {
              status.setText ("Please save file contents.");
              return;
          }

          System.exit (0);
      }

      // If the user selected Save from the menu, attempt to
      // save the editor's contents to the current file.
      // Note, the Save menu command will be disabled until
      // there is a current file.

      if (e.getActionCommand ().equals ("Save"))
      {
          // If the changed flag is true, the file contents
          // component's contents have changed.  Therefore,
          // attempt to save to the current file.

          if (changed)
          {
              // If the file was successfully saved, reset the
              // changed flag.  If the file was not successfully
              // changed (e.g., the user did not have a disk in
              // the floppy disk drive), give the user another
              // chance to save the editor's contents to the
              // current file by not reseting the changed flag.

              if (save (curfile))
                  changed = false;
          }

          // All done, so go away!

          return;
      }

      // If the user selected Open... or Save As... from the
      // menu, set the mode and title for the soon to be
      // displayed dialog box to the appropriate values.

      if (e.getActionCommand ().equals ("Open..."))
      {
          // There is no point in opening a new file if the
          // file contents component contains changed text.
          
          if (changed)
          {
              status.setText ("Please save file contents.");
              return;
          }

          mode = FileDialog.LOAD;
          title = "Open...";
      }
      else
      {
          mode = FileDialog.SAVE;
          title = "Save As...";
      }

      // Create and show either an Open... or Save As... dialog
      // box.

      FileDialog file = new FileDialog (this, title, mode);
      file.show ();

      // If the user selected a file from the dialog box for
      // loading or saving, try to load or save this file.

      String selectedFile;
      if ((selectedFile = file.getFile ()) != null)
      {
          // Build a complete path to this file by appending
          // the file name to directory information.

          String filename = file.getDirectory () + selectedFile;

          // Set the shape of the mouse cursor when it passes
          // over the file contents component to a wait cursor
          // symbol - to inform the user that a load or save is
          // taking place.

          Cursor c = new Cursor (Cursor.WAIT_CURSOR);
          fileContents.setCursor (c);

          // If the mode was load, do the load.  Otherwise, do the
          // save.

          if (mode == FileDialog.LOAD)
          {
              // If the load is successful, update the name of
              // the current file and the title in the title
              // bar.

              if (load (filename))
              {
                  curfile = new String (filename);
                  setTitle (TITLE + ": " + curfile);

                  Menu m = getMenuBar ().getMenu (0);
                  m.getItem (1).setEnabled (true);
                  m.getItem (2).setEnabled (true);
              }
          }
          else
          {
              // If the save is successful, update the name of
              // the current file and the title in the title
              // bar.

              if (save (filename))
              {
                  curfile = new String (filename);
                  setTitle (TITLE + ": " + curfile);
              }
          }

          // Set the shape of the mouse cursor when it passes
          // over the file contents component to a normal cursor
          // symbol - to inform the user that the load or save 
          // is finished.

          c = new Cursor (Cursor.DEFAULT_CURSOR);
          fileContents.setCursor (c);
      }
   }

   // Event handler for file contents changes.

   public void textValueChanged (TextEvent e)
   {
      changed = true;

      // Make sure that the Save As... menu item component is
      // enabled.

      Menu m = getMenuBar ().getMenu (0);
      m.getItem (2).setEnabled (true);
   }

   // File loading method.

   private boolean load (String filename)
   {
      try
      {
          File f = new File (filename);
          FileInputStream fis = new FileInputStream (f);

          int filesize = (int) f.length ();
          if (filesize > 32000)
          {
              status.setText ("File too big!");
              fis.close ();
              return false;
          }

          byte [] data = new byte [filesize];
          fis.read (data, 0, filesize);
          fis.close ();

          status.setText ("");

          fileContents.setText (new String (data));
      }
      catch (FileNotFoundException exc)
      {
          status.setText ("File Not Found: " + filename);
          return false;
      }
      catch (IOException exc)
      {
          status.setText ("IOException: " + filename);
          return false;
      }

      return true;
   }

   // File saving method.

   private boolean save (String filename)
   {
      byte [] data;

      File f  = new File (filename);

      try
      {
          FileOutputStream fos = new FileOutputStream (f);

          String text = fileContents.getText ();
          int textsize = text.length ();
          data = new byte [textsize];
          fos.write (text.getBytes ());
          fos.close ();

          changed = false;

          status.setText ("");
      }
      catch (IOException exc)
      {
          status.setText ("IOException: " + filename);
          return false;
      }

      return true;
   }
}
