// cm.java

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

// =========================================================
// Class: cm
//
// This class drives the contact manager.  It contains the
// main method which gets called as soon as this application
// begins to run.
// =========================================================

class cm extends Frame implements ActionListener
{
   // Container of contact objects (one object per business
   // contact).

   private Vector contacts = new Vector (100);

   // List of names component.  (Must specify java.awt in
   // front of List to distinguish the List class in the
   // java.awt package from the List class in the java.util
   // package.)

   private java.awt.List names = new java.awt.List ();

   // Delete and update button components.

   private Button delete;
   private Button update;

   // Default constructor.

   public cm ()
   {
      // Assign Contact Manager to title bar of frame window.

      super ("Contact Manager");

      // Add a listener that responds to window closing
      // events.  When this event occurs (by clicking on the
      // close box in the title bar), save contacts and exit.

      addWindowListener (new WindowAdapter ()
                         {
                             public void windowClosing
                                           (WindowEvent e)
                             {
                                saveContacts ();

                                System.exit (0);
                             }
                         });

      // Place an empty label in the north part of the frame
      // window.  This is done to correct an AWT positioning
      // problem.  (One thing that you'll come to realize as
      // you work with the AWT is that there are lots of bugs.)

      Label l = new Label ();
      add ("North", l);

      // Place the names component in the center part of the
      // frame window.

      add ("Center", names);

      // Create a panel object to hold four buttons.

      Panel p = new Panel ();
      Button b;

      // Add an Insert button to the Panel object and register
      // the current cm object as a listener for button events.

      p.add (b = new Button ("Insert"));
      b.addActionListener (this);

      // Add a Delete button to the Panel object and register
      // the current cm object as a listener for button events.

      p.add (delete = new Button ("Delete"));
      delete.addActionListener (this);

      // The Delete button should be disabled until there is at
      // least one contact to delete.

      delete.setEnabled (false);

      // Add an Update button to the Panel object and register
      // the current cm object as a listener for button events.

      p.add (update = new Button ("Update"));
      update.addActionListener (this);

      // The Update button should be disabled until there is at
      // least one contact to update.

      update.setEnabled (false);

      // Add a Finish button to the Panel object and register
      // the current cm object as a listener for button events.

      p.add (b = new Button ("Finish"));
      b.addActionListener (this);

      // Add the panel object to the frame window container.

      add ("South", p);

      // Set the background of the frame window container to
      // lightGray (to give a pleasing effect).

      setBackground (Color.lightGray);

      // Set the size of the frame window container to 400
      // pixels horizontally by 200 pixels vertically.

      setSize (400, 200);

      // Do not allow the user to resize the frame window.

      setResizable (false);

      // Load all contacts.

      loadContacts ();

      // Make sure that the frame window is visible.

      setVisible (true);
   }

   public void actionPerformed (ActionEvent e)
   {
      if (e.getActionCommand ().equals ("Delete"))
          delete ();
      else
      if (e.getActionCommand ().equals ("Finish"))
      {
          saveContacts ();
          System.exit (0);
      }
      else
      if (e.getActionCommand ().equals ("Insert"))
          insert ();
      else
          update ();
   }

   public Insets getInsets ()
   {
      // Return an Insets object that describes the number of
      // pixels to reserve as a border around the edges of the
      // frame window.

      return new Insets (10, 10, 10, 10);
   }

   public static void main (String [] args)
   {
      // Create a new cm object and let it do its thing.

      new cm ();
   }

   private void delete ()
   {
      // Obtain index of selected contact item from the names
      // component.

      int index = names.getSelectedIndex ();

      // If no item was selected, index is -1.  We cannot update
      // a contact if no contact item in the names component was
      // selected - because we would have nothing to work with.

      if (index != -1)
      {
          // Remove the contact item from the names component.

          names.remove (index);

          // Remove the Contact object from the contacts Vector
          // object.

          contacts.remove (index);

          // If there are no more contacts ...

          if (contacts.size () == 0)
          {
              delete.setEnabled (false);
              update.setEnabled (false);
          }
          else

          // Make sure that the first contact item in the names
          // list is highlighted.

          names.select (0);
      }
   }

   private void insert ()
   {
      // Create an Insert data entry form to enter information
      // for a new contact.

      DataEntryForm def = new DataEntryForm (this, "Insert");

      // If the bOk Boolean flag is set, this indicates the user
      // exited the form by pressing the Ok button.

      if (def.bOk)
      {
          // Create a Contact object and assign information from
          // the form to its fields.

          Contact temp = new Contact ();
          temp.fname = new String (def.fname.getText ());
          temp.lname = new String (def.lname.getText ());
          temp.phone = new String (def.phone.getText ());
          temp.fax = new String (def.fax.getText ());
          temp.email = new String (def.email.getText ());

          // Add a new contact item to the names component.

          names.add (temp.lname + ", " + temp.fname);

          // Add the Contact object to the contacts Vector
          // object.

          contacts.add (temp);

          // Make sure that the Delete and Update buttons are
          // enabled.

          delete.setEnabled (true);
          update.setEnabled (true);
      }

      // Destroy the dialog box.

      def.dispose ();

      // Make sure that the first contact item in the names list
      // is highlighted.

      names.select (0);
   }

   // ===========================================================
   // Load all contacts from contacts.dat into the contacts
   // Vector object.  Also, make sure that the last name/first
   // name from each contact is combined into a String object and
   // inserted into the names component - as a contact item.
   // ===========================================================

   private void loadContacts ()
   {
      FileInputStream fis = null;

      try
      {
          fis = new FileInputStream ("contacts.dat");

          DataInputStream dis = new DataInputStream (fis);

          int nContacts = dis.readInt ();

          for (int i = 0; i < nContacts; i++)
          {               
               Contact temp = new Contact ();

               temp.fname = dis.readUTF ();
               temp.lname = dis.readUTF ();
               temp.phone = dis.readUTF ();
               temp.fax = dis.readUTF ();
               temp.email = dis.readUTF ();

               names.add (temp.lname + ", " + temp.fname);
               contacts.add (temp);
          }

          if (nContacts > 0)
          {
              delete.setEnabled (true);
              update.setEnabled (true);
          }
      }
      catch (IOException e)
      {
      }
      finally
      {
         if (fis != null)
             try
             {
                 fis.close ();
             }
             catch (IOException e) {}
      }

      // Make sure that the first contact item in the names list
      // is highlighted.

      names.select (0);
   }

   // ========================================================
   // Save all Contact objects from the contacts Vector object
   // to contacts.dat.  The number of contacts are saved as an
   // int to make it easy for loadContacts () to do its job.
   // ========================================================

   private void saveContacts ()
   {
      FileOutputStream fos = null;

      try
      {
          fos = new FileOutputStream ("contacts.dat");

          DataOutputStream dos = new DataOutputStream (fos);

          dos.writeInt (contacts.size ());

          for (int i = 0; i < contacts.size (); i++)
          {
               Contact temp = (Contact) contacts.elementAt (i);

               dos.writeUTF (temp.fname);
               dos.writeUTF (temp.lname);
               dos.writeUTF (temp.phone);
               dos.writeUTF (temp.fax);
               dos.writeUTF (temp.email);
          }
      }
      catch (IOException e)
      {
         MsgBox mb = new MsgBox (this, "CM Error",
                                 e.toString ());
         mb.dispose ();
      }
      finally
      {
         if (fos != null)
             try
             {
                 fos.close ();
             }
             catch (IOException e) {}
      }
   }

   private void update ()
   {
      // Obtain index of selected contact item from the names
      // component.

      int index = names.getSelectedIndex ();

      // If no item was selected, index is -1.  We cannot update
      // a contact if no contact item in the names component was
      // selected - because we would have nothing to work with.

      if (index != -1)
      {
          // Obtain a reference to the Contact object (from the
          // contacts Vector object) that is associated with the
          // index.

          Contact temp = (Contact) contacts.elementAt (index);

          // Create and display an update entry form.

          DataEntryForm def = new DataEntryForm (this, "Update",
                                                 temp.fname,
                                                 temp.lname,
                                                 temp.phone,
                                                 temp.fax,
                                                 temp.email);

          // If the user pressed Ok...

          if (def.bOk)
          {
              // Update the contact information in the contacts
              // Vector object.

              temp.fname = new String (def.fname.getText ());
              temp.lname = new String (def.lname.getText ());
              temp.phone = new String (def.phone.getText ());
              temp.fax = new String (def.fax.getText ());
              temp.email = new String (def.email.getText ());

              // Make sure the screen reflects the update.

              names.replaceItem (temp.lname + ", " + temp.fname,
                                 index);
          }

          // Destroy the dialog box.

          def.dispose ();

          // Make sure that the first contact item in the names
          // list is highlighted.

          names.select (0);
       }
   }
}

// ========================================================
// Class: Contact
//
// This class describes the contents of a business contact.
// ========================================================

class Contact
{
   public String fname;
   public String lname;
   public String phone;
   public String fax;
   public String email;
}

// ==========================================================
// Class: DataEntryForm
//
// This class provides a data entry form for entering contact
// information.
// ==========================================================

class DataEntryForm extends Dialog implements ActionListener
{
   // bOk is a boolean flag.  When true, it indicates that
   // the Ok button was pressed to terminate the dialog box
   // (as opposed to the Cancel button).

   public boolean bOk;

   // The following components hold the text that the user
   // entered into the visible text fields.

   public TextField fname;
   public TextField lname;
   public TextField phone;
   public TextField fax;
   public TextField email;

   public void actionPerformed (ActionEvent e)
   {
      // If the user pressed the Ok button, indicate this
      // by assigning true to bOk.

      if (e.getActionCommand ().equals ("Ok"))
          bOk = true;

      // Destroy the dialog box and return to the point
      // just after the creation of the DataEntryForm object.

      dispose ();
   }

   public DataEntryForm (Frame parent, String title)
   {
      // Call the other constructor.  The current constructor
      // is used for insert operations.  The other constructor
      // is used for update operations.

      this (parent, title, "", "", "", "", "");      
   }

   public DataEntryForm (Frame parent, String title,
                         String fname, String lname,
                         String phone, String fax,
                         String email)
   {
      // Initialize the superclass layer.

      super (parent, title, true);

      // Choose a grid bag layout so that components can be more
      // accurately positioned.  (It looks nicer.)

      setLayout (new GridBagLayout ());

      // Add appropriate first name, last name, phone, fax, and
      // email components to the current DataEntryForm container.
      // (Remember, DataEntryForm is a subclass of Dialog.
      // Dialog is a container.  Therefore, DataEntryForm
      // inherits the ability to be a container.)

      addComponent (this, new Label ("First name: "), 0, 0, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.WEST);

      this.fname = new TextField (15);
      addComponent (this, this.fname, 1, 0, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      if (title.equals ("Update"))
          this.fname.setText (fname);

      addComponent (this, new Label ("Last name: "), 0, 1, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.WEST);

      this.lname = new TextField (15);
      addComponent (this, this.lname, 1, 1, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      if (title.equals ("Update"))
          this.lname.setText (lname);

      addComponent (this, new Label ("Phone number: "), 0, 2, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.WEST);

      this.phone = new TextField (15);
      addComponent (this, this.phone, 1, 2, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      if (title.equals ("Update"))
          this.phone.setText (phone);

      addComponent (this, new Label ("FAX number: "), 0, 3, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.WEST);

      this.fax = new TextField (15);
      addComponent (this, this.fax, 1, 3, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      if (title.equals ("Update"))
          this.fax.setText (fax);

      addComponent (this, new Label ("Email address: "), 0, 4, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.WEST);

      this.email = new TextField (15);
      addComponent (this, this.email, 1, 4, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      if (title.equals ("Update"))
          this.email.setText (email);

      addComponent (this, new Label (""), 0, 5, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      addComponent (this, new Label (""), 1, 5, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);

      Button b;

      // Add an Ok button to this container.

      addComponent (this, b = new Button ("Ok"), 0, 6, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);
      b.addActionListener (this);

      // Add a Cancel button to this container.

      addComponent (this, b = new Button ("Cancel"), 1, 6, 1, 1,
                    GridBagConstraints.NONE,
                    GridBagConstraints.CENTER);
      b.addActionListener (this);

      // Set the size of the dialog window to 250 pixels
      // horizontally by 200 pixels vertically.

      setSize (250, 200);

      // Do not allow users to resize the dialog window.

      setResizable (false);

      // Make sure that the dialog window is visible.

      setVisible (true);
   }

   private void addComponent (Container con, Component com,
                              int gridx, int gridy,
                              int gridw, int gridh, int fill,
                              int anchor)
      
   {
      // Get the current layout manager.  It is assumed to
      // be a GridBagLayout object.

      LayoutManager lm = con.getLayout ();

      // Create a GridBagConstraints object to make it
      // possible to customize component positioning.

      GridBagConstraints gbc = new GridBagConstraints ();

      // Assign the x and y grid positions.

      gbc.gridx = gridx;
      gbc.gridy = gridy;

      // Assign the number of grid blocks horizontally and
      // vertically that are occupied by the component.

      gbc.gridwidth = gridw;
      gbc.gridheight = gridh;

      // Specify the component's resize policy (fill) and
      // the direction in which the component is positioned
      // when its size is smaller than available space (anchor).

      gbc.fill = fill;
      gbc.anchor = anchor;

      // Set the new constraints that the grid bag layout
      // manager will use.

      ((GridBagLayout) lm).setConstraints (com, gbc);

      // Add the component to the container.

      con.add (com);
   }
}

// ===========================================================
// Class: MsgBox
//
// This class displays a message box to the user.  The message
// is usually an error message.  The user must press the Ok
// button to terminate the message box.
// ===========================================================

class MsgBox extends Dialog implements ActionListener
{
   public void actionPerformed (ActionEvent e)
   {
      // Terminate the dialog box in response to the user
      // pressing the Ok button.

      dispose ();
   }

   public MsgBox (Frame parent, String title, String msg)
   {
      // Initialize the superclass layer.

      super (parent, title, true);

      // Store the msg argument in a Label object and add
      // this object to the center part of the dialog window.

      Label l = new Label (msg);
      add ("Center", l);

      // Create a Button object and add it to the south part
      // of the dialog window.

      Button b = new Button ("Ok");
      add ("South", b);

      // Make the current object a listener to events that
      // occur as a result of the user pressing the Ok
      // button.

      b.addActionListener (this);

      // Make sure that the Ok button has the focus.

      b.requestFocus ();

      // Do not allow users to resize the dialog window.

      setResizable (false);

      // Allow the layout manager to choose an appropriate
      // size for the dialog window.

      pack ();

      // Make sure that the dialog window is visible.

      setVisible (true);
   }
}
